%{

#define YYSTYPE t_boxlik*
#include "logat.h"
#include <assert.h>
#if 0
#define DB(x) printf("yy:%s\n",(x));
#define DD(x) if(x)printf("yy:(%p,%p,%p)\n", (x)->fst, (x)->snd, (x)->trd);else printf("yy(--)\n");
#else

#define DB(x)
#define DD(x)

#endif

%}


%token IDENTIFIER CONSTANT STRING_LITERAL SIZEOF
%token PTR_OP INC_OP DEC_OP LEFT_OP RIGHT_OP LE_OP GE_OP EQ_OP NE_OP
%token AND_OP OR_OP MUL_ASSIGN DIV_ASSIGN MOD_ASSIGN ADD_ASSIGN
%token SUB_ASSIGN LEFT_ASSIGN RIGHT_ASSIGN AND_ASSIGN
%token XOR_ASSIGN OR_ASSIGN TYPE_NAME

%token TYPEDEF EXTERN STATIC AUTO REGISTER
%token CHAR SHORT INT LONG SIGNED UNSIGNED FLOAT DOUBLE CONST VOLATILE VOID
%token STRUCT UNION ENUM ELIPSIS RANGE

%token CASE DEFAULT IF ELSE SWITCH WHILE DO FOR GOTO CONTINUE BREAK RETURN

%start start_file
%%

primary_expr
        : identifier
		{ $$ = BX0(BX_IDENTIFIER); bxt_identifier($$, $1); }

        | CONSTANT
		{ $$ = $1; bxt_constant($$); }

        | STRING_LITERAL
		{ $$ = $1; $$->exprtyp = BX0(BX_POINTER); }

        | '(' expr ')'
		{ $$ = BX2(BX_EXPR, $2, $3); bxt_cptyp($$, $2); }
        ;

postfix_expr
        : primary_expr
		{ $$ = $1; }

        | postfix_expr '[' expr ']'
		{ $$ = BX3(BX_ARRAY, $1, $3, $4); bxt_array($$, $1, $3); }

        | postfix_expr '(' ')'
		{ $$ = BX1(BX_FCE, $1); bxt_fce($$, $1, NULL); }

        | postfix_expr '(' argument_expr_list ')'
		{ $$ = BX3(BX_FCE, $1, $3, $4); bxt_fce($$, $1, $3); }

        | postfix_expr '.' identifier
		{ $$ = BX2(BX_DOT, $1, $3); bxt_dot($$, $1, $3);}

        | postfix_expr PTR_OP identifier
		{ $$ = BX2(BX_PTR_OP, $1, $3); bxt_ptrop($$, $1, $3); }

        | postfix_expr INC_OP
		{ $$ = BX1(BX_INC_POST, $1); bxt_incdec($$, $1);}

        | postfix_expr DEC_OP
		{ $$ = BX1(BX_DEC_POST, $1); bxt_incdec($$, $1); }
        ;

argument_expr_list
        : assignment_expr
		{ $$ = BX2(BX_ARG_LIST, NULL, $1); }

        | argument_expr_list ',' assignment_expr
		{ $$ = BX2(BX_ARG_LIST, $1, $3); }
        ;

unary_expr
        : postfix_expr
		{ $$ = BX1(BX_EXPR, $1); bxt_cptyp($$, $1); }

        | INC_OP unary_expr
		{ $$ = BX1(BX_INC_PREF, $1); bxt_cptyp($$, $2); }

        | DEC_OP unary_expr
		{ $$ = BX1(BX_DEC_PREF, $1); bxt_cptyp($$, $2); }

        | unary_operator cast_expr
		{ $$ = BX2(BX_UNARY, $1, $2); bxt_unary($$, $2); }

        | SIZEOF unary_expr
		{ $$ = BX1(BX_SIZEOF_EXPR, $1);
		  bxt_toint($1);
		  bxt_cptyp($$, $1);
		}

        | SIZEOF '(' type_name ')'
		{ $$ = BX1(BX_SIZEOF_EXPR, $1);
		  bxt_toint($1);
		  bxt_cptyp($$, $1);
		}
        ;

unary_operator
        : '&'
		{ $$ = $1; } 
        | '*'
		{ $$ = $1; } 
        | '+'
		{ $$ = $1; } 
        | '-'
		{ $$ = $1; } 
        | '~'
		{ $$ = $1; } 
        | '!'
		{ $$ = $1; } 
        ;

cast_expr
        : unary_expr
		{ $$ = $1; }
        | '(' type_name ')' cast_expr
		{ $$ = BX2(BX_CAST, $2, $4); bxt_cast($$, $2, $4); }
        ;

multiplicative_expr
        : cast_expr
		{ $$ = $1; }

        | multiplicative_expr '*' cast_expr
		{ $$ = BX3(BX_MUL, $1, $2, $3); bxt_binop($$, $1, $3); }

        | multiplicative_expr '/' cast_expr
		{ $$ = BX3(BX_DIV, $1, $2, $3); bxt_binop($$, $1, $3); }

        | multiplicative_expr '%' cast_expr
		{ $$ = BX3(BX_MOD, $1, $2,  $3); bxt_binop($$, $1, $3); }
        ;

additive_expr
        : multiplicative_expr
		{ $$ = $1; }

        | additive_expr '+' multiplicative_expr
		{ $$ = BX3(BX_ADD, $1, $2, $3); bxt_binop($$, $1, $3); }

        | additive_expr '-' multiplicative_expr
		{ $$ = BX3(BX_SUB, $1, $2, $3); bxt_binop($$, $1, $3); }
        ;

shift_expr
        : additive_expr
		{ $$ = $1; }

        | shift_expr LEFT_OP additive_expr
		{ $$ = BX3(BX_LSHIFT, $1, $2, $3); bxt_binop($$, $1, $3); }

        | shift_expr RIGHT_OP additive_expr
		{ $$ = BX3(BX_RSHIFT, $1, $2, $3); bxt_binop($$, $1, $3); }
        ;

relational_expr
        : shift_expr
		{ $$ = $1; }

        | relational_expr '<' shift_expr
		{ $$ = BX3(BX_LT, $1, $2, $3); bxt_binop($$, $1, $3); }

        | relational_expr '>' shift_expr
		{ $$ = BX3(BX_GT, $1, $2, $3); bxt_binop($$, $1, $3); }

        | relational_expr LE_OP shift_expr
		{ $$ = BX3(BX_LE, $1, $2, $3); bxt_binop($$, $1, $3); }

        | relational_expr GE_OP shift_expr
		{ $$ = BX3(BX_GE, $1, $2, $3); bxt_binop($$, $1, $3); }

        ;

equality_expr
        : relational_expr
		{ $$ = $1; }

        | equality_expr EQ_OP relational_expr
		{ $$ = BX2(BX_EQ, $1, $3); bxt_binop($$, $1, $3); }

        | equality_expr NE_OP relational_expr
		{ $$ = BX2(BX_NEQ, $1, $3); bxt_binop($$, $1, $3); }
        ;

and_expr
        : equality_expr
		{ $$ = $1; }
        | and_expr '&' equality_expr
		{ $$ = BX2(BX_AND, $1, $3); bxt_binop($$, $1, $3); }
        ;

exclusive_or_expr
        : and_expr
		{ $$ = $1; }
        | exclusive_or_expr '^' and_expr
		{ $$ = BX2(BX_XOR, $1, $3); bxt_binop($$, $1, $3); }
        ;

inclusive_or_expr
        : exclusive_or_expr
		{ $$ = $1; }
        | inclusive_or_expr '|' exclusive_or_expr
		{ $$ = BX2(BX_OR, $1, $3); bxt_binop($$, $1, $3); }
        ;

logical_and_expr
        : inclusive_or_expr
		{ $$ = $1; }
        | logical_and_expr AND_OP inclusive_or_expr
		{ $$ = BX2(BX_LAND, $1, $3); bxt_binop($$, $1, $3); }
        ;

logical_or_expr
        : logical_and_expr
		{ $$ = $1; }
        | logical_or_expr OR_OP logical_and_expr
		{ $$ = BX2(BX_LOR, $1, $3); bxt_binop($$, $1, $3); }
        ;

conditional_expr
        : logical_or_expr
		{ $$ = $1; }
        | logical_or_expr '?' logical_or_expr ':' conditional_expr
		{ $$ = BX3(BX_COND, $1, $3, $5); bxt_cond($$); }
        ;

assignment_expr
        : conditional_expr
		{ $$ = $1; }
        | unary_expr assignment_operator assignment_expr
		{ $$ = BX3(BX_ASSIG, $1, $2, $3); bxt_assig($$); }
        ;

assignment_operator
        : '='
		{ $$ = BX1(BX_ASSIG_EQ, $1); }
        | MUL_ASSIGN
		{ $$ = BX1(BX_ASSIG_MUL, $1); }
        | DIV_ASSIGN
		{ $$ = BX1(BX_ASSIG_DIV, $1); }
        | MOD_ASSIGN
		{ $$ = BX1(BX_ASSIG_MOD, $1); }
        | ADD_ASSIGN
		{ $$ = BX1(BX_ASSIG_ADD, $1); }
        | SUB_ASSIGN
		{ $$ = BX1(BX_ASSIG_SUB, $1); }
        | LEFT_ASSIGN
		{ $$ = BX1(BX_ASSIG_LEFT, $1); }
        | RIGHT_ASSIGN
		{ $$ = BX1(BX_ASSIG_RIGHT, $1); }
        | AND_ASSIGN
		{ $$ = BX1(BX_ASSIG_AND, $1); }
        | XOR_ASSIGN
		{ $$ = BX1(BX_ASSIG_XOR, $1); }
        | OR_ASSIGN
		{ $$ = BX1(BX_ASSIG_OR, $1); }
        ;

expr
        : assignment_expr
		{ $$ = $1; DB("expr"); }
        | expr ',' assignment_expr
		{ $$ = BX2(BX_EXPR_LIST, $1, $2); bxt_cptyp($$, $3); }
        ;

constant_expr
        : conditional_expr
		{ $$ = $1; }
        ;

declaration
        : declaration_specifiers ';'
		{ $$ = BX1(BX_DECLARATION, $1); bx_addtype($$); }

        | declaration_specifiers init_declarator_list ';'
		{ $$ = BX2(BX_DECLARATION, $1, $2); 
		  bx_addtype($$);
 		  bx_id_addspecifier($1, $2);
		}
        ;

declaration_specifiers
        : storage_class_specifier
		{ $$ = BX_DECLSPEC($1, NULL,  NULL); }

        | storage_class_specifier declaration_specifiers
		{ $$ = BX_DECLSPEC($1, NULL, $2); }

        | type_specifier
		{ $$ = BX_DECLSPEC(NULL, $1, NULL); }

        | type_specifier declaration_specifiers
		{ $$ = BX_DECLSPEC(NULL, $1, $2); }
        ;

init_declarator_list
        : init_declarator
		{ $$ = BX1(BX_DECLARATOR, $1); }

        | init_declarator_list ',' init_declarator
		{ $$ = $1;  BX_LAST($$, BX1(BX_DECLARATOR,  $3)); }
        ;

init_declarator
        : declarator
		{ $$ = $1; }

        | declarator '=' initializer
		{ $$ = $1; }
        ;

storage_class_specifier
        : TYPEDEF
		{ $$ = BX0(BX_TOK_TYPEDEF); }
        | EXTERN
		{ $$ = BX0(BX_TOK_EXTERN); }
        | STATIC
		{ $$ = BX0(BX_TOK_STATIC); }
        | AUTO
		{ $$ = BX0(BX_TOK_AUTO); }
        | REGISTER
		{ $$ = BX0(BX_TOK_REGISTER); }
        ;

type_specifier
        : CHAR
		{ $$ = BX0(BX_TOK_CHAR); }

        | SHORT
		{ $$ = BX0(BX_TOK_SHORT); }

        | INT
		{ $$ = BX0(BX_TOK_INT); }

        | LONG
		{ $$ = BX0(BX_TOK_LONG); }

        | SIGNED
		{ $$ = BX0(BX_TOK_SIGNED); }

        | UNSIGNED
		{ $$ = BX0(BX_TOK_UNSIGNED); }

        | FLOAT
		{ $$ = BX0(BX_TOK_DOUBLE); }

        | DOUBLE
		{ $$ = BX0(BX_TOK_LOGAT);
		  /* quich hack */
		  $1->txt = "logat";
		  /*float does not exist enymore */
		}

        | CONST
		{ $$ = BX0(BX_TOK_CONST); }

        | VOLATILE
		{ $$ = BX0(BX_TOK_VOLATILE); }

        | VOID
		{ $$ = BX0(BX_TOK_VOID); }

        | struct_or_union_specifier
		{ $$ = $1; }

        | enum_specifier
		{ $$ = $1; }

        | TYPE_NAME
		{ $$ = BX1(BX_TYPE_NAME, $1); }
        ;

struct_or_union_specifier
        : struct_or_union identifier '{' struct_declaration_list '}'
		{ $$ = BX3(BX_STRUCT_OR_UNION_SPEC, $1, $2, $4); }

        | struct_or_union '{' struct_declaration_list '}'
		{ $$ = BX3(BX_STRUCT_OR_UNION_SPEC, $1, NULL, $3); }

        | struct_or_union identifier
		{ $$ = BX3(BX_STRUCT_OR_UNION_SPEC, $1, $2, NULL); }
        ;

struct_or_union
        : STRUCT
		{ $$ = BX0(BX_TOK_STRUCT); }
        | UNION
		{ $$ = BX0(BX_TOK_UNION); }
        ;

struct_declaration_list
        : struct_declaration
		{ $$ = BX1(BX_STRUCT_DECL_LIST, $1); }

        | struct_declaration_list struct_declaration
		{ $$ = $1; BX_LAST($$, BX1(BX_STRUCT_DECL_LIST, $2)); }
        ;

struct_declaration
        : type_specifier_list struct_declarator_list ';'
		{
		  t_boxlik *tmp;
		  tmp = BX_DECLSPEC(NULL, $1, NULL);
		  $$ = BX2(BX_STRUCT_DECL, tmp, $2);
		  bx_id_addspecifier(tmp, $2);
		}

        ;

struct_declarator_list
        : struct_declarator
		{ $$ = BX1(BX_STRUCT_DECLARATOR, $1); }
        | struct_declarator_list ',' struct_declarator
		{ $$ = $1; BX_LAST($$, BX1(BX_STRUCT_DECLARATOR, $3)); }
        ;

struct_declarator
        : declarator
		{ $$ = $1; }

        | ':' constant_expr
		{ $$ = BX2(BX_STRUCT_DECLARATOR, NULL, $2); }

        | declarator ':' constant_expr
		{ $$ = BX2(BX_STRUCT_DECLARATOR, $1, $3); }
        ;

enum_specifier
        : ENUM '{' enumerator_list '}'
		{ $$ = BX2(BX_ENUM_SPEC, NULL, $3); }

        | ENUM identifier '{' enumerator_list '}'
		{ $$ = BX2(BX_ENUM_SPEC, $2, $4); }

        | ENUM identifier
		{ $$ = BX1(BX_ENUM_SPEC, $2); }
        ;

enumerator_list
        : enumerator
		{ $$ = $1; }

        | enumerator_list ',' enumerator
		{ $$ = $1;  BX_LAST($$, $3); }

        ;

enumerator
        : identifier
		{ $$ = BX1(BX_ENUMERATOR, $1); }

        | identifier '=' constant_expr
		{ $$ = BX2(BX_ENUMERATOR, $1, $3); }
        ;

declarator
        : declarator2
		{ $$ = $1; }

        | pointer declarator2
		{ $$ = $2; bx_id_addpoiter($$, $1); }

        ;

declarator2
        : identifier
		{ $$ = bxt_identifier_decl($1); }

        | '(' declarator ')'
		{ $$ = $2; }

        | declarator2 '[' ']'
		{ $$ = $1; bx_id_addtype($$, BX0(BX_POINTER)); }

        | declarator2 '[' constant_expr ']'
		{ $$ = $1; bx_id_addtype($$, BX0(BX_POINTER)); }

        | declarator2 '(' ')'
		{ $$ = $1; bx_id_addtype($$, BX0(BX_TYPE_FCE));}

        | declarator2 '(' parameter_type_list ')'
		{ $$ = $1; bx_id_addtype($$, BX2(BX_TYPE_FCE, NULL, $3));}

        | declarator2 '(' parameter_identifier_list ')'
		{ $$ = $1; bx_id_addtype($$, BX2(BX_TYPE_FCE, NULL, $3));}
        ;

pointer
        : '*'
		{ $$= BX0(BX_POINTER); }

        | '*' type_specifier_list
		{ $$= BX0(BX_POINTER); }

        | '*' pointer
		{ $$= BX1(BX_POINTER, $2); }

        | '*' type_specifier_list pointer
		{ $$= BX1(BX_POINTER, $3); }
        ;

type_specifier_list
        : type_specifier
		{ $$ = $1; }

        | type_specifier_list type_specifier
		{ $$ = $1; BX_LAST($$, $2); }
        ;

parameter_identifier_list
        : identifier_list
		{ $$ = $1; }

        | identifier_list ',' ELIPSIS
		{ $$ = $1; printf("elipsis not yet!!!!!!!!!\n\n"); }
        ;

identifier_list
        : identifier
		{ $$ =  BX2(BX_PARAM_LIST, NULL, $1); }

        | identifier_list ',' identifier
		{ $$ =  BX2(BX_PARAM_LIST, $1, $3); }
        ;

parameter_type_list
        : parameter_list
		{ $$ = $1; }

        | parameter_list ',' ELIPSIS
		{ $$ = $1; printf("elipsis not yet!!!!!!!!!\n\n"); }
        ;

parameter_list
        : parameter_declaration
		{ $$ =  BX2(BX_PARAM_LIST, NULL, $1); }

        | parameter_list ',' parameter_declaration
		{ $$ =  BX2(BX_PARAM_LIST, $1, $3); }
        ;

parameter_declaration
        : type_specifier_list declarator
		{
		  $$ = BX2(BX_PAR_DECL, $1, $2);
		  bx_id_truetype($2, $1);
		  bx_id_addlocally($2);
		}

        | type_name
		{ $$ = $1; }
        ;

type_name
        : type_specifier_list
		{ $$ = BX1(BX_ADECLARATOR, $1); }

        | type_specifier_list abstract_declarator
		{ $$ = $2; bx_id_truetype($2, $1); }
        ;

abstract_declarator
        : pointer
		{ $$ = BX0(BX_ADECLARATOR); bx_id_addpoiter($$, $1); }

        | abstract_declarator2
		{ $$ = $1; }

        | pointer abstract_declarator2
		{ $$ = $2; bx_id_addpoiter($$, $1); }
        ;

abstract_declarator2
        : '(' abstract_declarator ')'
		{ $$ = BX1(BX_DECL2_INPAR, $2); }

        | '[' ']'
		{ $$ = BX0(BX_DECL2_BRA); }

        | '[' constant_expr ']'
		{ $$ = BX2(BX_DECL2_BRA, NULL, $2); }

        | abstract_declarator2 '[' ']'
		{ $$ = BX1(BX_DECL2_BRA, $1); }

        | abstract_declarator2 '[' constant_expr ']'
		{ $$ = BX2(BX_DECL2_BRA, $1, $3); }

        | '(' ')'
		{ $$ = BX0(BX_DECL2_FCE); }

        | '(' parameter_type_list ')'
		{ $$ = BX2(BX_DECL2_FCE, NULL, $2); }

        | abstract_declarator2 '(' ')'
		{ $$ = BX1(BX_DECL2_FCE, $1); }

        | abstract_declarator2 '(' parameter_type_list ')'
		{ $$ = BX2(BX_DECL2_FCE, $1, $2); }
        ;

initializer
        : assignment_expr
		{ $$ = $1; }
        | '{' initializer_list '}'
		{ $$ = BX1(BX_INITIALIZER, $2); }

        | '{' initializer_list ',' '}'
		{ $$ = BX1(BX_INITIALIZER, $2); }
        ;

initializer_list
        : initializer
		{ $$ = $1; }

        | initializer_list ',' initializer
		{ $$ = $1; BX_LAST($$, $3); }
        ;


statement
        : labeled_statement
		{ $$ = $1; }

        | compound_statement
		{ $$ = $1; }

        | expression_statement
		{ $$ = $1; }

        | selection_statement
		{ $$ = $1; }

        | iteration_statement
		{ $$ = $1; }

        | jump_statement
		{ $$ = $1; }
        ;

labeled_statement
        : identifier ':' statement
		{ $$ = $3; }
        | CASE constant_expr ':' statement
		{ $$ = $4; }
        | DEFAULT ':' statement
		{ $$ = $3; }
        ;

compound_statement
        : '{' '}'
		{ $$ = NULL; }
        | '{' statement_list '}'
		{ $$ = BX2(BX_COMPOUND,NULL, $2); }

        | '{' declaration_list '}'
		{ $$ = BX1(BX_COMPOUND, $2); }

        | '{' declaration_list statement_list '}'
		{ $$ = BX2(BX_COMPOUND, $2, $3); }
        ;

declaration_list
        : declaration
		{ $$ = $1; }
        | declaration_list declaration
		{ $$ = $1; BX_LAST($$, $2); }
        ;

statement_list
        : statement
		{ $$ = $1; }
        | statement_list statement
		{ $$ = BX2(BX_STATEMENT, $1, $2); }
        ;

expression_statement
        : ';'
        | expr ';'
		{ $$ = $1; DD($$); DB("exp_stat"); }
        ;

selection_statement
        : IF '(' expr ')' statement
		{ $$ = BX2(BX_IF, $3, $5); }

        | IF '(' expr ')' statement ELSE statement
		{ $$ = BX3(BX_IF, $3, $5, $7); }

        | SWITCH '(' expr ')' statement
		{ $$ = BX2(BX_IF, $3, $5); }
        ;

iteration_statement
        : WHILE '(' expr ')' statement
		{ $$ = BX2(BX_IF, $3, $5); }

        | DO statement WHILE '(' expr ')' ';'
		{ $$ = BX2(BX_IF, $3, $5); }

        | FOR '(' ';' ';' ')' statement
		{ $$ = BX2(BX_IF, NULL, $6); }

        | FOR '(' ';' ';' expr ')' statement
		{ $$ = BX2(BX_IF, $5, $7); }

        | FOR '(' ';' expr ';' ')' statement
		{ $$ = BX2(BX_IF, $4, $7); }

        | FOR '(' ';' expr ';' expr ')' statement
		{ $$ = BX2(BX_IF, BX2(BX_EXPR_LIST, $4, $6 ), $8); }

        | FOR '(' expr ';' ';' ')' statement
		{ $$ = BX2(BX_IF, $3, $7); }

        | FOR '(' expr ';' ';' expr ')' statement
		{ $$ = BX2(BX_IF, BX2(BX_EXPR_LIST, $4, $6 ), $8); }

        | FOR '(' expr ';' expr ';' ')' statement
		{ $$ = BX2(BX_IF, BX2(BX_EXPR_LIST, $3, $5 ), $8); }

        | FOR '(' expr ';' expr ';' expr ')' statement
		{ $$ = BX2(BX_IF, BX3(BX_EXPR_LIST, $3, $5, $4), $9); }
        ;

jump_statement
        : GOTO identifier ';'
        | CONTINUE ';'
        | BREAK ';'
        | RETURN ';'
        | RETURN expr ';'
        ;

start_file
	: file
		{ $$ = $1; boxlik_root = $$; DD($$); DB("start_file"); } 
	;

file
        : external_definition			
		{ $$ = $1; DD($$); DB("fileee");}

        | file external_definition	
		{ $$ = BX2(BX_DEF_LIST, $1, $2); }
        ;

external_definition
        : function_definition
		{ $$ = $1; DD($$); DB("exter_defini");}
        | declaration
		{ $$ = $1; }
        ;

function_definition
        : declarator
		{ BX_INFCE(1); }
	  function_body
		{
		  $$ = BX3(BX_FCE_DEF, NULL, $1, $3);
		  DD($$); DB("func_decl"); 
		  assert($$ != NULL);
		  BX_INFCE(0);
		  BX_CLTBL;

		}

        | declaration_specifiers declarator
		{ BX_INFCE(1); }
	  function_body
		{
		  $$ = BX3(BX_FCE_DEF, $1, $2, $4);
		  DD($$);
		  DB("func_decl"); 
		  assert($$ != NULL);
		  BX_INFCE(0);
		  BX_CLTBL;
		}
        ;

function_body
        : compound_statement
		{ $$ = $1; DD($$); DB("function_body"); }

        | declaration_list compound_statement
		{ $$ = BX2(BX_FCE_BODY, $1, $2); }
        ; 

identifier
        : IDENTIFIER
		{ $$ = $1; }
        ;
%%
#include <stdio.h>

extern char yytext[];
extern int column;
extern char last_line[100];
extern int last_line_top;


yyerror(s)
char *s;
{
        fflush(stdout);
        printf("%s", last_line);
        printf("\n%*s\n%*s\n", column, "^", column, s);
}
